"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const electron_1 = require("electron");
const child_process_1 = require("child_process");
const path_1 = __importDefault(require("path"));
const name = "Stop Chrome";
const version = "2.0.11";
const icon = path_1.default.join(__dirname, "../", "assets", "icon.png");
const green = path_1.default.join(__dirname, "../", "assets", "state_green.png");
const red = path_1.default.join(__dirname, "../", "assets", "state_red.png");
const delay = 60;
const timeout = 10;
class Main {
    static main() {
        return __awaiter(this, void 0, void 0, function* () {
            if (require('electron-squirrel-startup') || !electron_1.app.requestSingleInstanceLock())
                return electron_1.app.quit();
            electron_1.app.once("ready", () => {
                Main.tray = new electron_1.Tray(green);
                Main.tray.setToolTip(name);
                Main.tray.setImage(icon);
                Main.tray.on("click", () => Main.tray.popUpContextMenu());
                Main.tray.on("mouse-move", Main.checkChromeProcess);
                Main.tray.setContextMenu(Main.menu = electron_1.Menu.buildFromTemplate([
                    {
                        label: `${name} v${version}`,
                        type: "normal",
                        icon: electron_1.nativeImage.createFromPath(icon).resize({ width: 16, height: 16 }),
                        enabled: false
                    },
                    {
                        type: "separator"
                    },
                    {
                        id: "chrome",
                        label: "Stop Chrome",
                        type: "normal",
                        enabled: false,
                        click: Main.endChromeProcess
                    },
                    {
                        type: "separator"
                    },
                    {
                        id: "quit",
                        label: "Quit",
                        type: "normal",
                        click: () => {
                            Main.tray.destroy();
                            electron_1.app.quit();
                            process.exit(0);
                        }
                    }
                ]));
                Main.checkChromeProcess();
                setInterval(Main.checkChromeProcess, delay * 1000);
            });
        });
    }
    static checkChromeProcess() {
        if (Main.isChecking)
            return;
        Main.isChecking = true;
        Main.lookup()
            .then(processes => {
            let chromePID = null;
            for (const process of processes) {
                if (process.name == "chrome.exe") {
                    const parentProcess = Main.find(processes, process.ppid);
                    if (parentProcess && parentProcess.name == "chrome.exe") {
                        chromePID = parentProcess.pid;
                        break;
                    }
                }
            }
            Main.chromePID = chromePID;
            Main.menu.getMenuItemById("chrome").enabled = !!chromePID;
            Main.tray.setToolTip(`Chrome is${chromePID ? "" : " not"} running`);
            Main.tray.setImage(chromePID ? red : green);
        })
            .finally(() => {
            Main.isChecking = false;
        });
    }
    static endChromeProcess() {
        if (Main.chromePID) {
            const index = electron_1.dialog.showMessageBoxSync({
                title: name,
                message: "Are you sure you want to stop chrome?",
                type: "warning",
                buttons: ["Yes", "No"],
                defaultId: 1,
                cancelId: 1,
                noLink: true
            });
            if (index == 0)
                try {
                    process.kill(Main.chromePID);
                }
                catch (error) {
                }
                finally {
                    for (let i = 0; i <= 10; i += 2)
                        setTimeout(() => {
                            if (Main.chromePID)
                                Main.checkChromeProcess();
                        }, i * 1000);
                }
        }
    }
    static lookup() {
        return new Promise((resolve, reject) => {
            (0, child_process_1.exec)(`Get-CimInstance -Class Win32_Process -Filter "Name='chrome.exe'" | Select-Object Name,ProcessId,ParentProcessId`, { shell: "powershell.exe", timeout: 1000 * timeout }, (err, stdout, stderr) => {
                if (!err) {
                    let iname = null;
                    let ipid = null;
                    let ippid = null;
                    const processes = [];
                    for (const line of stdout.trim().split(/\r*\n/)) {
                        const values = line.trim().split(/\s+/);
                        if (iname == null || ipid == null || ippid == null) {
                            iname = values.indexOf("Name");
                            ipid = values.indexOf("ProcessId");
                            ippid = values.indexOf("ParentProcessId");
                        }
                        else if (!values[0].startsWith('-') && !values[1].startsWith('-') && !values[2].startsWith('-'))
                            processes.push({
                                name: values[iname],
                                pid: parseInt(values[ipid]),
                                ppid: parseInt(values[ippid])
                            });
                    }
                    resolve(processes);
                }
                resolve([]);
            });
        });
    }
    static find(processes, pid) {
        for (const process of processes)
            if (process.pid == pid)
                return process;
        return null;
    }
}
Main.chromePID = null;
Main.isChecking = false;
process.on("unhandledRejection", (error, promise) => {
    console.error(`Unhandled rejection at:\n  Promise ${promise}\n  ${error.stack}`);
    if (electron_1.app) {
        if (Main.tray)
            Main.tray.destroy();
        electron_1.app.quit();
        process.exit(-1);
    }
});
Main.main().catch((error) => {
    console.error(error.stack);
    if (electron_1.app) {
        if (Main.tray)
            Main.tray.destroy();
        electron_1.app.quit();
        process.exit(-1);
    }
});
//# sourceMappingURL=main.js.map